/*
  Copyright (c) 2013, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#include "iface.hpp"
#include "mmss.hpp"
#include "node.hpp"


using namespace MMSS;

extern "C" {

gmrf_addr_t gmrf_iface_get_addr(gmrf_t *gmrf, gmrf_iface_t *iface) {
	return *(static_cast<iface_t*>(iface)->get_address());
}

const char* gmrf_iface_get_name(gmrf_t *gmrf, gmrf_iface_t *iface) {
	return static_cast<iface_t*>(iface)->get_name().c_str();
}

size_t gmrf_iface_get_mtu(gmrf_t *gmrf, gmrf_iface_t *iface) {
	return static_cast<iface_t*>(iface)->get_network()->mtu;
}

bool gmrf_iface_send(gmrf_t *gmrf, gmrf_iface_t *iface, const void *data, size_t len, const gmrf_addr_t *dest) {
	auto src_iface = static_cast<iface_t*>(iface)->shared_from_this();

	for (auto dest_iface : src_iface->get_network()->interfaces) {
		if (gmrf_addr_equal(dest_iface->get_address(), dest)) {
			enqueue(static_cast<node_t*>(gmrf)->get_context(), src_iface, dest_iface, data, len);
			break;
		}
	}

	return true;
}

bool gmrf_iface_send_bc(gmrf_t *gmrf, gmrf_iface_t *iface, const void *data, size_t len) {
	auto src_iface = static_cast<iface_t*>(iface)->shared_from_this();

	for (auto dest_iface : src_iface->get_network()->interfaces) {
		if (dest_iface != src_iface)
			enqueue(static_cast<node_t*>(gmrf)->get_context(), src_iface, dest_iface, data, len);
	}

	return true;
}

void gmrf_schedule(gmrf_t *gmrf, gmrf_scheduled_func f, void *arg, unsigned delay) {
	node_t *node = static_cast<node_t*>(gmrf);

	std::shared_ptr<scheduled_t> scheduled = std::make_shared<scheduled_t>();

	scheduled->node = node->shared_from_this();
	scheduled->f = f;
	scheduled->arg = arg;

	node->get_context()->event_queue.put(std::move(scheduled), node->get_context()->now()+delay);
}

gmrf_time_t gmrf_now(gmrf_t *gmrf) {
	return static_cast<node_t*>(gmrf)->get_context()->now();
}

void gmrf_random_bytes(gmrf_t *gmrf, void *buffer, size_t len) {
	uint8_t *data = reinterpret_cast<uint8_t*>(buffer);

	size_t i;
	for (i = 0; i < len; i++)
		data[i] = static_cast<node_t*>(gmrf)->random_byte();
}

void gmrf_logf(gmrf_t *gmrf, int priority, const char *format, ...) {
	node_t *node = static_cast<node_t*>(gmrf);

	va_list ap;
	va_start(ap, format);
	node->get_context()->vlogf_orig(node, priority, format, ap);
	va_end(ap);
}

}
